@@ -11,6 +11,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import sys
+import os
 import logging
 
 import botocore.session
@@ -515,10 +516,11 @@ class CLIOperationCaller(object):
         # for credentials so we can give a good error message.
         if not self._session.get_credentials():
             raise NoCredentialsError()
+        verify = self._resolve_verify_var(parsed_globals.no_verify_ssl)
         endpoint = operation_object.service.get_endpoint(
             region_name=parsed_globals.region,
-            endpoint_url=parsed_globals.endpoint_url)
-        endpoint.verify = not parsed_globals.no_verify_ssl
+            endpoint_url=parsed_globals.endpoint_url,
+            verify=verify)
         if operation_object.can_paginate and parsed_globals.paginate:
             pages = operation_object.paginate(endpoint, **parameters)
             self._display_response(operation_object, pages,
@@ -530,6 +532,14 @@ class CLIOperationCaller(object):
                                    parsed_globals)
         return 0
 
+    def _resolve_verify_var(self, no_verify_ssl):
+        verify = None
+        if no_verify_ssl:
+            verify = False
+        else:
+            verify = os.environ.get('AWS_CA_BUNDLE')
+        return verify
+
     def _display_response(self, operation, response, args):
         output = args.output
         if output is None:
